{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvButton.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvButton;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, StdCtrls, Menus, Controls,
  Forms;

type
  TJvButton = class(TButton)
  private
    FOnMouseEnter: TNotifyEvent;
    FColor, FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged : TNotifyEvent;
    FDropMenu: TPopupMenu;
    FHotTrack: boolean;
    FHotFont: TFont;
    FFontSave: TFont;
    FOver: Boolean;
    procedure SetHotFont(const Value: TFont);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
  public
    procedure Click; override;
    procedure CreateParams(var Params: TCreateParams); override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property DropDownMenu:TPopupMenu read FDropMenu write FDropMenu;
    property HotTrack: boolean read FHotTrack write FHotTrack default false;
    property HotTrackFont: TFont read FHotFont write SetHotFont;

    property HintColor:TColor read FColor write FColor default clInfoBk;
    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FOnMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

{**************************************************}
constructor TJvButton.Create(AOwner: TComponent);
begin
  inherited;
  FColor := clInfoBk;
  FHotTrack := false;
  FHotFont := TFont.Create;
  FFontSave := TFont.Create;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FOver := false;
end;
{**************************************************}
destructor TJvButton.Destroy;
begin
  FHotFont.Free;
  FFontSave.Free;
  inherited;
end;
{**************************************************}
procedure TJvButton.Click;
begin
  inherited;
  if FDropMenu<>nil then
  begin
    FDropMenu.Popup(GetClientOrigin.x,GetClientOrigin.y+self.height);
    Perform(CM_MOUSELEAVE,0,0);
  end;
end;
{**************************************************}
procedure TJvButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.style := Params.style or BS_MULTILINE;
end;
{**************************************************}
procedure TJvButton.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvButton.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvButton.CMMouseEnter(var Message:TMessage);
begin
  if not FOver then
  begin
    FSaved:=Application.HintColor;
    Application.HintColor:=FColor;
    if FHotTrack then
    begin
      FFontSave.Assign(Font);
      Font.Assign(FHotFont);
    end;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvButton.CMMouseLeave(var Message:TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FHotTrack then
      Font.Assign(FFontSave);
    FOver := false;
  end;

  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(self);
end;
{**************************************************}
procedure TJvButton.SetHotFont(const Value: TFont);
begin
  FHotFont.Assign(Value);
end;
{**************************************************}
end.
