{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvClock.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvClock;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvTypes, extctrls;

type
  TClockstyle = (csTime,csTimeDate,csDateTime,csDate);
  {$EXTERNALSYM TClockstyle}
  TOnUpdate = procedure (Sender: TObject;Value:string) of object;
  {$EXTERNALSYM TOnUpdate}

  TJvClock = class(TWinControl)
  private
    FLabel:TLabel;
    FPanel:TPanel;
    Ftimer:TTimer;
    FOnUpdate: TOnUpdate;
    FOnMouseEnter : TNotifyEvent;
    FColor,FSaved:TColor;
    FOnMouseLeave : TNotifyEvent;
    FOver:boolean;
    FClock: TClockstyle;
    function GetFont: TFont;
    procedure SetFont(const Value: TFont);
    procedure OnUpdate(Sender: TObject);
    procedure Setclock(const Value: TClockstyle);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Font:TFont read GetFont write SetFont;
    property OnTime:TOnUpdate read FOnUpdate write FOnUpdate;
    property HintColor:TColor read FColor write FColor default clInfoBk;
    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property ClockStyle:TClockstyle read FClock write SetClock default csTime;
    property ShowHint;
    property Visible;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnResize;
  end;

implementation

{*******************************************************}
constructor TJvClock.Create(AOwner: TComponent);
begin
  inherited;
  FColor := clInfoBk;
  FOver := false;
  FClock := csTime;

  self.width := 100;
  self.height := 50;
  FPanel := TPanel.Create(self);
  FPanel.Parent := self;
  FPanel.visible := true;
  FPanel.Align := AlClient;
  Fpanel.BevelInner := bvLowered;
  FPanel.BevelOuter := bvRaised;
  FPanel.BevelWidth := 2;

  FLabel := TLabel.Create(FPanel);
  FLabel.Caption := TimeToStr(Time);
  FLabel.parent := FPanel;
  Flabel.AutoSize := true;
  Flabel.Left := (self.width div 2)-(Flabel.width div 2);
  Flabel.top := (self.height div 2)-(Flabel.height div 2);
  FLabel.visible := true;

  Ftimer := TTimer.Create(Self);
  FTimer.OnTimer := OnUpdate;
  FTimer.Interval := 910;
  Ftimer.enabled := true;
end;
{*******************************************************}
destructor TJvClock.Destroy;
begin
  FTImer.free;
  FLabel.Free;
  FPanel.Free;
  inherited;
end;
{*******************************************************}
function TJvClock.GetFont: TFont;
begin
  result := Flabel.font;
end;
{*******************************************************}
procedure TJvClock.OnUpdate(Sender: TObject);
begin
  case FClock of
    csTime: Flabel.caption:=TimeToStr(time);
    csTimeDate: FLabel.caption:=TimeToStr(time)+' '+DatetoStr(Date);
    csDateTime: FLabel.caption:=DatetoStr(Date)+' '+TimeToStr(time);
    csDate: FLabel.caption:=DatetoStr(Date);
  end;
  if Assigned(FOnUpdate) then
    FonUpdate(self,Flabel.caption);
end;
{*******************************************************}
procedure TJvClock.SetFont(const Value: TFont);
begin
  Flabel.font := Value;
  Flabel.Left := (self.width div 2)-(Flabel.width div 2);
  Flabel.top := (self.height div 2)-(Flabel.height div 2);
end;
{*******************************************************}
procedure TJvClock.WMSize(var Message: TWMSize);
begin
  inherited;
  Flabel.Left := (self.width div 2)-(Flabel.width div 2);
  Flabel.top := (self.height div 2)-(Flabel.height div 2);
end;
{*******************************************************}
procedure TJvClock.CMMouseEnter(var Message: TMessage);
begin
  FOver := true;
  FSaved := Application.HintColor;
  Application.HintColor := FColor;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvClock.CMMouseLeave(var Message: TMessage);
begin
  Application.HintColor := FSaved;
  FOver := false;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
procedure TJvClock.Setclock(const Value: TClockstyle);
begin
  FCLock := Value;
  OnUpdate(self);
  Flabel.Left := (self.width div 2)-(Flabel.width div 2);
  Flabel.top := (self.height div 2)-(Flabel.height div 2);
end;
{**************************************************}
end.
