{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCommStatus.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCommStatus;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes;

type
  TJvCommWatcher = class(TThread)
  private
  protected
    procedure Execute; override;
    procedure Changed;
  public
    FHandle:THandle;
    FStat:Cardinal;
    FOnChange:TNotifyEvent;
  end;

  TJvCommStatus = class(TComponent)
  private
    FClear: boolean;
    FDataSet: boolean;
    FRing: boolean;
    FReceive: boolean;
    FHandle:THandle;
    FWatcher:TJvCommWatcher;
    FBidon: boolean;
    FComm: TJvCommPort;
    FOnChanged: TNotifyEvent;
    procedure SetComm(const Value: TJvCommPort);
    procedure OnChange(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property ClearToSend:boolean read FClear write FBidon;
    property DataSetReady:boolean read FDataSet write FBidon;
    property Ring:boolean read FRing write FBidon;
    property ReceiveLine:boolean read FReceive write FBidon;
    property Comm:TJvCommPort read FComm write SetComm default cpNOCOM;
    property OnChanged:TNotifyEvent read FOnChanged write FOnChanged;
  end;

implementation

{**************************************************}
constructor TJvCommStatus.Create(AOwner: TComponent);
begin
   inherited;
   FComm:=cpNOCOM;
   FHandle:=0;

   if not(csDesigning in componentstate) then
   begin
      FWatcher:=TJvCommWatcher.Create(true);
      FWatcher.FreeOnTerminate:=true;

      FWatcher.FHandle:=FHandle;
      FWatcher.FStat:=0;
      FWatcher.FonChange:=OnChange;

      FWatcher.Resume;
   end
   else FWatcher:=nil;

   SetComm(FComm);
end;
{**************************************************}
destructor TJvCommStatus.Destroy;
begin
   if FWatcher<>nil then
   begin
      FWatcher.Terminate;
      FWatcher:=nil;
   end;
   if FHandle<>0 then
      CloseHandle(FHandle);
   inherited;
end;
{**************************************************}
procedure TJvCommStatus.OnChange(Sender: TObject);
var
  stat:cardinal;
begin
   if (FWatcher<>nil) and (FHandle<>0) then
      stat:=FWatcher.FStat
   else
      stat:=0;

   FClear:=(stat and MS_CTS_ON)<>0;
   FDataSet:=(stat and MS_DSR_ON)<>0;
   FRing:=(stat and MS_RING_ON)<>0;
   FReceive:=(stat and MS_RLSD_ON)<>0;

   if Assigned(FOnChanged) then
      FOnChanged(self);
end;
{**************************************************}
procedure TJvCommStatus.SetComm(const Value: TJvCommPort);
var
  stat:cardinal;
begin
   if FWatcher<>nil then
      FWatcher.FHandle:=0;
   FComm := Value;
   if FHandle<>0 then
      CloseHandle(FHandle);

   case FComm of
       cpNOCOM : FHandle:=0;
       cpCOM1 : FHandle:=CreateFile('COM1',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM2 : FHandle:=CreateFile('COM2',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM3 : FHandle:=CreateFile('COM3',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM4 : FHandle:=CreateFile('COM4',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM5 : FHandle:=CreateFile('COM5',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM6 : FHandle:=CreateFile('COM6',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM7 : FHandle:=CreateFile('COM7',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
       cpCOM8 : FHandle:=CreateFile('COM8',0,FILE_SHARE_READ or FILE_SHARE_WRITE,nil,OPEN_EXISTING,0,0);
   end;

   FClear:=false;
   FDataSet:=false;
   FRing:=false;
   FReceive:=false;

   if GetCommModemStatus(FHandle,stat) then
   begin
      FClear:=(stat and MS_CTS_ON)<>0;
      FDataSet:=(stat and MS_DSR_ON)<>0;
      FRing:=(stat and MS_RING_ON)<>0;
      FReceive:=(stat and MS_RLSD_ON)<>0;
   end;

   if FWatcher<>nil then
   begin
      FWatcher.FHandle:=FHandle;
      FWatcher.FStat:=0;
      if FHandle<>0 then
        FWatcher.Resume
      else
        FWatcher.Suspend;
   end;
   OnChange(self);
end;
{**************************************************}

///////////////////////////////////////////////////////////
// TJvCommWatcher
///////////////////////////////////////////////////////////

{**************************************************}
procedure TJvCommWatcher.Changed;
begin
   FOnChange(nil);
end;
{**************************************************}
procedure TJvCommWatcher.Execute;
var
  mask:Cardinal;
begin
   while not(Terminated) do
   begin
      if FHandle<>0 then
      begin
         GetCommModemStatus(FHandle,mask);
         if mask<>FStat then
         begin
            FStat:=mask;
            Synchronize(Changed);
         end;
      end;
      sleep(50);
   end;
end;
{**************************************************}
end.
