{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCustomBox.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCustomBox;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes, StdCtrls,
  Buttons, ActnList;

type
  TPosition = (onLeft,onRight);

  TJvCustomEditOptions = class(TPersistent)
  private
    FEnabled: boolean;
    FCtl3d: boolean;
    FReadOnly: boolean;
    FShowHint: boolean;
    FMaxLength: Integer;
    FHint: string;
    FColor: TColor;
    FFont: TFont;
    FOnChange: TNotifyEvent;
    FText: string;
    FCursor: TCursor;
    procedure SetColor(const Value: TColor);
    procedure SetCtl3d(const Value: boolean);
    procedure SetEnabled(const Value: boolean);
    procedure SetFont(const Value: TFont);
    procedure SetHint(const Value: string);
    procedure SetMaxLength(const Value: integer);
    procedure SetReadOnly(const Value: boolean);
    procedure SetShowHint(const Value: boolean);
    procedure SetText(const Value: string);
    procedure SetCursor(const Value: TCursor);
  protected
    property OnChanged: TNotifyEvent read FOnChange write FOnChange;
    procedure Changed;
  public
    constructor Create;
    destructor Destroy;override;
  published
    property Text: string read FText write SetText;
    property ReadOnly: boolean read FReadOnly write SetReadOnly default false;
    property Font: TFont read FFont write SetFont;
    property Hint: string read FHint write SetHint;
    property Ctl3d: boolean read FCtl3d write SetCtl3d default true;
    property ShowHint: boolean read FShowHint write SetShowHint default false;
    property Color: TColor read FColor write SetColor default clWindow;
    property MaxLength: Integer read FMaxLength write SetMaxLength default 0;
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Cursor: TCursor read FCursor write SetCursor default crDefault;
  end;

  TJvCustomButtonOptions = class(TPersistent)
  private
    FFlat: boolean;
    FEnabled: boolean;
    FTransparent: boolean;
    FShowHint: boolean;
    FHint: string;
    FCaption: string;
    FGlyph: TBitmap;
    FFont: TFont;
    FOnChange: TNotifyEvent;
    FPosition: TPosition;
    FWidth: Integer;
    FCursor: TCursor;
    procedure SetCaption(const Value: string);
    procedure SetEnabled(const Value: boolean);
    procedure SetFlat(const Value: boolean);
    procedure SetFont(const Value: TFont);
    procedure SetGlyph(const Value: TBitmap);
    procedure SetHint(const Value: string);
    procedure SetPosition(const Value: TPosition);
    procedure SetShowHint(const Value: boolean);
    procedure SetTransparent(const Value: boolean);
    procedure SetWidth(const Value: integer);
    procedure SetCursor(const Value: TCursor);
  protected
    property OnChanged: TNotifyEvent read FOnChange write FOnChange;
    procedure Changed;
    procedure GlyphChanged(Sender: TObject);
  public
    constructor Create;
    destructor Destroy;override;
  published
    property Flat: boolean read FFlat write SetFlat default false;
    property Hint: string read FHint write SetHint;
    property ShowHint: boolean read FShowHint write SetShowHint default false;
    property Font: TFont read FFont write SetFont;
    property Enabled: boolean read FEnabled write SetEnabled default true;
    property Caption: string read FCaption write SetCaption;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Position: TPosition read FPosition write SetPosition default OnRight;
    property Transparent: boolean read FTransparent write SetTransparent default true;
    property Width: Integer read FWidth write SetWidth default 20;
    property Cursor: TCursor read FCursor write SetCursor default crDefault;
  end;

  TJvCustomBox = class(TWinControl)
  private
    FEditOptions: TJvCustomEditOptions;
    FButtonOptions: TJvCustomButtonOptions;
    FEdit: TEdit;
    FButton: TSpeedButton;
    procedure EditChanged(Sender: TObject);
    procedure ButtonChanged(Sender: TObject);
  protected
    procedure BtnClick(Sender: TObject);virtual;
    procedure EditChange(Sender: TObject);virtual;
    function GetEdit: TEdit;
    function GetButton: TSpeedButton;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Execute;virtual;
  published
    property Align;
    property ShowHint;
    property BorderWidth;
    property Color;
    property Visible;
    property ParentShowHint;
    property Enabled;
    property OnKeyDown;
    property OnKeyPress;
    property OnEnter;
    property OnExit;
    property OnDockDrop;
    property OnDockOver;
    property OnGetSiteInfo;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnUnDock;
    property PopupMenu;
    property TabOrder;
    property TabStop;
    property BevelEdges;
    property BevelInner;
    property BevelKind;
    property BevelOuter;
    property BevelWidth;

    property Edit:TJvCustomEditOptions read FEditOptions write FEditOptions;
    property Button:TJvCustomButtonOptions read FButtonOptions write FButtonOptions;
  end;

implementation

{$R RES_CustomBox.res}

{*****************************************************}
procedure TJvCustomBox.BtnClick(Sender: TObject);
begin
  //Do nothing
end;
{*****************************************************}
procedure TJvCustomBox.ButtonChanged(Sender: TObject);
begin
  FButton.Font.Assign(FButtonOptions.Font);
  FButton.Glyph.Assign(FButtonOptions.Glyph);
  FButton.Flat := FButtonOptions.Flat;
  FButton.Hint := FButtonOptions.Hint;
  FButton.ShowHint := FButtonOptions.ShowHint;
  FButton.Enabled := FButtonOptions.Enabled;
  FButton.Caption := FButtonOptions.Caption;
  FButton.Transparent := FButtonOptions.Transparent;
  FButton.Width := FButtonOptions.Width;
  FButton.Cursor := FButtonOptions.Cursor;
  if FButtonOptions.FPosition = OnLeft then
    FButton.Align := alLeft
  else
    FButton.Align := alRight;
end;
{*****************************************************}
constructor TJvCustomBox.Create(AOwner: TComponent);
begin
  inherited;
  FEditOptions := TJvCustomEditOptions.Create;
  FButtonOptions := TJvCustomButtonOptions.Create;

  FButton := TSpeedButton.Create(self);
  with FButton do
  begin
    Glyph := TBitmap.Create;
    Parent := self;
    Align := alRight;
    Width := 20;
    OnClick := BtnClick;
  end;

  FEdit := TEdit.Create(self);
  with FEdit do
  begin
    Parent := self;
    Align := alClient;
    OnChange := EditChange;
  end;

  Width := 150;
  Height := 20;

  FButtonOptions.OnChanged := ButtonChanged;
  FEditOptions.OnChanged := EditChanged;
end;
{*****************************************************}
destructor TJvCustomBox.Destroy;
begin
  FEditOptions.Free;
  FButtonOptions.Free;
  FButton.Free;
  FEdit.Free;
  inherited;
end;
{*****************************************************}
procedure TJvCustomBox.EditChange(Sender: TObject);
begin
  FEditOptions.Text := FEdit.Text;
end;
{*****************************************************}
procedure TJvCustomBox.EditChanged(Sender: TObject);
begin
  FEdit.Text := FEditOptions.Text;
  FEdit.ReadOnly := FEditOptions.ReadOnly;
  FEdit.Hint := FEditOptions.Hint;
  FEdit.Ctl3d := FEditOptions.Ctl3d;
  FEdit.ShowHint := FEditOptions.ShowHint;
  FEdit.MaxLength := FEditOptions.MaxLength;
  FEdit.Enabled := FEditOptions.Enabled;
  FEdit.Color := FEditOptions.Color;
  FEdit.Cursor := FEditOptions.Cursor;
  FEdit.Font.Assign(FEditOptions.Font);
  FEdit.OnChange := EditChange;
end;
{*****************************************************}
procedure TJvCustomBox.Execute;
begin
  BtnClick(self);
end;
{*****************************************************}
function TJvCustomBox.GetButton: TSpeedButton;
begin
  result := FButton;
end;
{*****************************************************}
function TJvCustomBox.GetEdit: TEdit;
begin
  result := FEdit;
end;
{*****************************************************}

{ TJvCustomEditOptions }

{*****************************************************}
procedure TJvCustomEditOptions.Changed;
begin
  if Assigned(FOnChange) then
    FOnChange(self);
end;
{*****************************************************}
constructor TJvCustomEditOptions.Create;
begin
  FReadOnly := false;
  FFont := TFont.Create;
  FCtl3d := true;
  FShowHint := false;
  FColor := clWindow;
  FMaxLength := 0;
  FEnabled := true;
  FCursor := crDefault;
end;
{*****************************************************}
destructor TJvCustomEditOptions.Destroy;
begin
  FFont.Free;
  inherited;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetColor(const Value: TColor);
begin
  FColor := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetCtl3d(const Value: boolean);
begin
  FCtl3d := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetCursor(const Value: TCursor);
begin
  FCursor := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetHint(const Value: string);
begin
  FHint := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetMaxLength(const Value: integer);
begin
  FMaxLength := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetReadOnly(const Value: boolean);
begin
  FReadOnly := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetShowHint(const Value: boolean);
begin
  FShowHint := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomEditOptions.SetText(const Value: string);
begin
  FText := Value;
  Changed;
end;
{*****************************************************}

{ TJvCustomButtonOptions }

{*****************************************************}
procedure TJvCustomButtonOptions.Changed;
begin
  if Assigned(FOnChange) then
    FOnChange(self);
end;
{*****************************************************}
constructor TJvCustomButtonOptions.Create;
begin
  FFlat := false;
  FShowHint := false;
  FEnabled := true;
  FPosition := OnRight;
  FTransparent := true;
  FFont := TFont.Create;
  FGlyph := TBitmap.Create;
  FWidth := 20;
  FCursor := crDefault;
  FGlyph.OnChange := GlyphChanged;
end;
{*****************************************************}
destructor TJvCustomButtonOptions.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.GlyphChanged(Sender: TObject);
begin
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetCaption(const Value: string);
begin
  FCaption := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetCursor(const Value: TCursor);
begin
  FCursor := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetFlat(const Value: boolean);
begin
  FFlat := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetGlyph(const Value: TBitmap);
begin
  FGlyph.Assign(Value);
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetHint(const Value: string);
begin
  FHint := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetPosition(const Value: TPosition);
begin
  FPosition := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetShowHint(const Value: boolean);
begin
  FShowHint := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetTransparent(const Value: boolean);
begin
  FTransparent := Value;
  Changed;
end;
{*****************************************************}
procedure TJvCustomButtonOptions.SetWidth(const Value: integer);
begin
  FWidth := Value;
  Changed;
end;
{*****************************************************}

end.
