{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvEdit.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvEdit;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, Forms,
  JvPropAutoSave, JvMaxPixel, Dialogs;

type
  TJvEdit = class(TEdit)
  private
    FAlignment: TAlignment;
    FAutoCtl3D: boolean;
    FOver: boolean;
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnRestored: TNotifyEvent;
    FAutoSave: TJvAutoSave;
    FMaxPixel: TJvMaxPixel;
    procedure SetAutoCtl3D(const Value: boolean);
    procedure SetAlignment(Value: TAlignment);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
    procedure Change; override;
    procedure MaxPixelChanged(Sender: TObject);
  public
    procedure CreateParams(var Params: TCreateParams); Override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Loaded;override;
  published
    function IsEmpty:boolean;

    property AutoSave: TJvAutoSave read FAutoSave write FAutoSave;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property Align;
    property HintColor: TColor read FColor write FColor default clInfoBk;
    property HotTrack: boolean read FAutoCtl3D write SetAutoCtl3D default false;
    property Modified;
    property MaxPixel: TJvMaxPixel read FMaxPixel write FMaxPixel;
    property SelStart;
    property SelText;
    property SelLength;

    property OnMouseEnter: TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave: TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged: TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange: TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnRestored: TNotifyEvent read FOnRestored write FOnRestored;
  end;

implementation

{***********************************************}
constructor TJvEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Parent := TWinControl(AOwner);
  FColor := clInfoBk;
  FAutoctl3d := false;
  FOver := false;
  FAlignment := taLeftJustify;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FAutoSave := TJvAutoSave.Create(self);
  FMaxPixel := TJvMaxPixel.Create(self);
  FMaxPixel.OnChanged := MaxPixelChanged;
end;
{***********************************************}
procedure TJvEdit.Loaded;
var
 st: string;
begin
  inherited;
  if (FAutoSave.LoadValue(st))  then
  begin
    Text := st;
    if Assigned(FOnRestored) then
      FOnRestored(self);
  end;
end;
{***********************************************}
destructor TJvEdit.Destroy;
begin
  FAutoSave.Free;
  FMaxPixel.Free;
  inherited;
end;
{***********************************************}
procedure TJvEdit.Change;
var
 st: string;
begin
  inherited;
  st := Text;
  FMaxPixel.Test(st,Font);
  if st<>Text then
    Text := st;
  SelStart := Length(Text);
  FAutoSave.SaveValue(Text);
end;
{***********************************************}
procedure TJvEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if Parent<>nil then
   case FAlignment of
     taLeftJustify  : Params.Style := Params.Style or ES_LEFT;
     taRightJustify : Params.Style := Params.Style or ES_RIGHT;
     taCenter       : Params.Style := Params.Style or ES_CENTER;
   end;
end;
{***********************************************}
procedure TJvEdit.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if FAutoCtl3d then
      Ctl3d := true;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvEdit.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FAutoCtl3d then
      Ctl3d := false;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
procedure TJvEdit.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvEdit.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvEdit.SetAutoCtl3D(const Value: boolean);
begin
  FAutoCtl3D := Value;
  if FAutoCtl3d then
    Ctl3d := false;
end;
{**************************************************}
function TJvEdit.IsEmpty:boolean;
begin
  result := length(caption)=0;
end;
{***********************************************}
procedure TJvEdit.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    ReCreateWnd;
  end;
end;
{***********************************************}
procedure TJvEdit.MaxPixelChanged(Sender: TObject);
var
 st: string;
begin
  st := Text;
  FMaxPixel.Test(st,Font);
  if st<>Text then
    Text := st;
  SelStart := Length(Text);
end;
{***********************************************}
end.
