{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFormToHtml.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFormToHtml;

{$ObjExportAll On} 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TJvFormToHtml = class(TComponent)
  private
    ts:TstringList;
  protected
  public
  published
    procedure FormToHtml(Form:TForm;path:string);
  end;

implementation

{*************************************************}
function FontToCss(font:TFont):string;
begin
   result:=format(';font-size:%d;color:#%d;font-weight:',[font.size,font.color]);
   if (fsbold in font.style) then result:=result+'bold;' else result:=result+'normal;';
   result:=result+'font-family:'+font.name;
end;
{*************************************************}
procedure TJvFormToHtml.FormToHtml(Form:TForm;path:string);
var
   i,j:Integer;
   c:TComponent;
   st:string;
begin
   ts:=TstringList.Create;
   ts.add('<HTML><BODY>');
   for i:=0 to form.ComponentCount-1 do
   begin
      c:=form.Components[i];
      st:='';
      if c is TLabel then
      begin
         st:=format('<LABEL style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as tlabel).left,(c as tlabel).top,(c as tlabel).height,(c as tlabel).width]);
         st:=st+FontToCss((c as Tlabel).font)+'"';
         st:=st+' TITLE="'+(c as tlabel).hint+'"';
         st:=st+' NAME='+(c as tlabel).Name;
         st:=st+'>';
         st:=st+(c as tlabel).caption+'</LABEL>';
      end
      else if c is TButton then
      begin
         st:=format('<JvTTON style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TButton).left,(c as TButton).top,(c as TButton).height,(c as TButton).width]);
         st:=st+FontToCss((c as TButton).font)+'"';
         st:=st+' TITLE="'+(c as TButton).hint+'"';
         st:=st+' TABORDER='+IntToStr((c as TButton).TabOrder);
         st:=st+' NAME='+(c as TButton).Name;
         if (c as TButton).enabled=false then st:=st+' DISABLED';
         st:=st+'>';
         st:=st+(c as TButton).caption+'</BUTTON>';
      end
      else if c is TMemo then
      begin
         st:=format('<TEXTAREA style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TMemo).left,(c as TMemo).top,(c as TMemo).height,(c as TMemo).width]);
         st:=st+FontToCss((c as TMemo).font)+'"';
         st:=st+' TITLE="'+(c as TMemo).hint+'"';
         if (c as TMemo).ReadOnly then st:=st+' ReadOnly';
         if (c as TMemo).enabled=false then st:=st+' DISABLED';
         st:=st+' NAME='+(c as TMemo).Name;
         st:=st+' TABORDER='+IntToStr((c as TMemo).TabOrder);
         if (c as TMemo).WordWrap then st:=st+' WRAP=PHYSICAL' else st:=st+' WRAP=OFF';
         st:=st+'>';
         st:=st+(c as TMemo).text+'</TEXTAREA>';
      end
      else if c is TCheckBox then
      begin
         st:=format('<INPUT style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TCheckBox).left,(c as TCheckBox).top,(c as TCheckBox).height,10]);
         st:=st+FontToCss((c as TCheckBox).font)+'"';
         st:=st+' TITLE="'+(c as TCheckBox).hint+'"';
         if (c as TCheckBox).enabled=false then st:=st+' DISABLED';
         if (c as TCheckBox).checked then st:=st+' CHECKED';
         st:=st+' TABORDER='+IntToStr((c as TCheckBox).TabOrder);
         st:=st+' NAME='+(c as TCheckBox).Name;
         st:=st+' TYPE="CHECKBOX">';
         ts.add(st);
         st:=format('<LABEL style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TCheckBox).left+13,(c as TCheckBox).top,(c as TCheckBox).height,(c as TCheckBox).width]);
         st:=st+FontToCss((c as TCheckBox).font)+'"';
         st:=st+' TITLE="'+(c as TCheckBox).hint+'"';
         st:=st+'>';
         st:=st+(c as TCheckBox).caption+'</LABEL>';
      end
      else if c is TRadioButton then
      begin
         st:=format('<INPUT style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TRadioButton).left,(c as TRadioButton).top,(c as TRadioButton).height,10]);
         st:=st+FontToCss((c as TRadioButton).font)+'"';
         st:=st+' TITLE="'+(c as TRadioButton).hint+'"';
         if (c as TRadioButton).enabled=false then st:=st+' DISABLED';
         if (c as TRadioButton).checked then st:=st+' CHECKED';
         st:=st+' NAME='+(c as TRadioButton).Parent.Name;
         st:=st+' TABORDER='+IntToStr((c as TRadioButton).TabOrder);
         st:=st+' TYPE="RADIO">';
         ts.add(st);
         st:=format('<LABEL style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TRadioButton).left+13,(c as TRadioButton).top,(c as TRadioButton).height,(c as TRadioButton).width]);
         st:=st+FontToCss((c as TRadioButton).font)+'"';
         st:=st+' TITLE="'+(c as TRadioButton).hint+'"';
         st:=st+'>';
         st:=st+(c as TRadioButton).caption+'</LABEL>';
      end
      else if c is TEdit then
      begin
         st:=format('<INPUT TYPE="TEXT" style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TEdit).left,(c as TEdit).top,(c as TEdit).height,(c as TEdit).width]);
         st:=st+FontToCss((c as TEdit).font)+'"';
         st:=st+' TITLE="'+(c as TEdit).hint+'"';
         st:=st+' TABORDER='+IntToStr((c as TEdit).TabOrder);
         st:=st+' NAME='+(c as TEdit).Name;
         if (C as TEdit).ReadOnly then st:=st+' ReadOnly';
         if (C as Tedit).MaxLength<>0 then st:=st+' MAXLENGTH='+IntToStr((C as Tedit).maxlength);
         if (c as TEdit).enabled=false then st:=st+' DISABLED';
         st:=st+' Value='+(c as TEdit).text;
         st:=st+'>';
      end
      else if c is TCombobox then
      begin
         st:=format('<SELECT style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TCombobox).left,(c as TCombobox).top,(c as TCombobox).height,(c as TCombobox).width]);
         st:=st+FontToCss((c as TCombobox).font)+'"';
         st:=st+' TITLE="'+(c as TCombobox).hint+'"';
         st:=st+' TABORDER='+IntToStr((c as TCombobox).TabOrder);
         st:=st+' NAME='+(c as TCombobox).Name;
         if (c as TCombobox).enabled=false then st:=st+' DISABLED';
         st:=st+'>';
         ts.add(st);
         for j:=0 to (c as TCombobox).items.count-1 do
         begin
            if (c as TCombobox).ItemIndex=j then
                ts.add('<OPTION SELECTED>'+(c as TCombobox).items[j])
            else
                ts.add('<OPTION>'+(c as TCombobox).items[j]);
         end;
         st:='</SELECT>';
      end
      else if c is TListbox then
      begin
         st:=format('<SELECT style="position:absolute;left:%d;top:%d;height:%d;width:%d',[(c as TListbox).left,(c as TListbox).top,(c as TListbox).height,(c as TListbox).width]);
         st:=st+FontToCss((c as TListbox).font)+'"';
         st:=st+' MULTIPLE TITLE="'+(c as TListbox).hint+'"';
         st:=st+' TABORDER='+IntToStr((c as TListbox).TabOrder);
         st:=st+' NAME='+(c as TListbox).Name;
         if (c as TListbox).enabled=false then st:=st+' DISABLED';
         st:=st+'>';
         ts.add(st);
         for j:=0 to (c as TListbox).items.count-1 do
         begin
            if (c as TListbox).ItemIndex=j then
                ts.add('<OPTION SELECTED>'+(c as TListbox).items[j])
            else
                ts.add('<OPTION>'+(c as TListbox).items[j]);
         end;
         st:='</SELECT>';
      end;

      if st<>'' then ts.add(st);
   end;
   ts.add('</BODY></HTML>');
   ts.SaveToFile(path);
   ts.free;
end;
{*************************************************}
end.
