{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvHMru.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvHMru;

{*******************************************************}
{   This unit is an interface to the MRU List (comctl32)}
{   Informations from :                                 }
{      http://www.geocities.com/SiliconValley/4942      }
{*******************************************************}

interface

uses
   Windows, Messages, JvFunctions;

type
  {if the MRUF_string_LIST flag is specified, list items are stored in the
  registry as string Values. if the MRUF_BINARY_LIST flag is specified, items
  are stored as binary data. For string lists, the comparison function
  should accept two LPCSTR parameters (the two strings to be compared).
  For binary lists, the comparison function should accept three
  parameters - two LPCVOID parameters (the data for the two items) and
  a DWORD specifying the number of bytes to compare.

  The MRUF_DELAYED_SAVE flag determines how often the list order data is saved
  to the registry. Normally it would be saved whenever you add an item to the
  list, but if the MRUF_DELAYED_SAVE flag is specified when the list is
  constructed, the order data is only saved when the list handle is
  finally freed. You free the list using the FreeMRUList function. This will free
  any internal data structures used by the list, as well as flushing the
  unsaved order data if necessary.}
  MRUComparestring = function (lpszstring1,lpszstring2:PChar):Integer;
  MRUCompareData   = function (lpData1, lpData2:pointer; cbData:DWORD):Integer;

  {cbSize specifies the size of the structure.
  nMaxItems specifies the maximum number of items allowed in the list.
  dwFlags specifies the  type of list to Create.
  hKey identifies the handle of the root registry key under which the list information will be stored.
  lpszSubKey points to a null-terminated string specifying the name of the registry subkey.
  lpfnCompare specifies the function that is used when comparing items in the list.}
  TMruList = record
      cbSize : DWORD;
      nMaxItems : DWORD;
      dwFlags : DWORD;
      hKey : HKEY;
      lpszSubKey : PChar;
      case boolean of
         true  : (lpfnComparestring : MRUComparestring;);
         false : (lpfnCompareData   : MRUCompareData;);
  end;
  PMruList = ^TMruList;

const
  MRUF_string_LIST  = 0;
  MRUF_BINARY_LIST  = 1;
  MRUF_DELAYED_SAVE = 2;

type
  {You open an MRU list, or Create a new list using the CreateMRUList function.
  You pass it a structure with information about the list,  and it will return
  a list handle which you will use in all subsequent functions.}
  TCreateMRUList = function (lpCreateInfo:PMruList):THandle;stdcall;
  TFreeMruList = procedure (hList:THandle);stdcall;


  {You can add an item to the list using the AddMRUstring function or the
  AddMRUData function . AddMRUstring passes the new item in as a string.
  AddMRUData passes the item in as a chunk of binary data - you specify a
  pointer to a buffer and the number of bytes in the buffer.
        if the item being added already exists, it isn't added again - its
  position is just moved  to the front of the list order. if the list is full,
  the least recently used item in the list is replaced with the new item.
  The return Value is a number corresponding to the registry name where the
  item is stored. 0 corresponds with the 'a' Value, 1 corresponds with
  'b', etc. if there was an error adding the item, the function returns -1.}
  TAddMrustring = function (hList:THandle;lpszstring:pchar):Integer;stdcall;
  TAddMruData = function (hList:THandle;lpData:pointer;cbData:DWORD):Integer;stdcall;


  {  To remove an item from an MRU list, you can use the DelMRUstring function,
  passing it the position in the list of the item you wish to remove (0 specifies
  the most recently used item - this is not the same as the return Value
  from the AddMRUxxx functions). DelMRUstring can also be used for removing
  items from binary lists - there isn't a corresponding DelMRUData function,
  although you could easily #define an alias for the sake of consistency.
     Unfortunately, that isn't the end of the story. Due to a bug in the DelMRUstring
  implementation, if you attempt to add a new item to the list after removing
  an item, you may end up corrupting memory or causing your Application to crash.
  To  work around this, you should always close the list and
  reopen it again before you attempt any other operations on
  the list.
     DelMRUstring returns TRUE if the item was removed
  successfully. if the specified position is out of range, the
  return Value is FALSE.}
  TDelMRUstring = function (hList:THandle;nItemPos:integer):boolean;stdcall;

  {You can enumerate through the items in a list with the
  EnumMRUList function. nItemPos is the position in the list of the item
  you wish to retrieve (0 being the most recently used item).
  lpBuffer points to a buffer where the item will be stored.
  nBufferSize specifies the buffer size (for string lists the size is in
  characters; for binary lists it's in bytes).
        The return Value specifies how many bytes were copied to the buffer in
  the case  of a binary list (this may be less than the full size of the item
  if the buffer is too small). For string lists, the return Value is the full
  length of the string being retrieved (which may be longer than the
  actual string returned if the buffer is too small). if you enumerate past
  the end of the list, the function returns -1.
        You can also determine the exact number of items in a list by specifying
  a NULL buffer pointer, or by specifiying a negative Value for nItemPos.
  The return Value is then the number of items in the list.}
  TEnumMRUList = function (hList:THandle;nItemPos:Integer;lpBuffer:pointer;nBufferSize:DWord):Integer;stdcall;

  {if you need to search for a particular item in the list, you should use the
  functions FindMRUstring and FindMRUData. Much like the AddMRUxxx functions,
  FindMRUstring specifies the item as a string and FindMRUData passes it in as
  binary data in a buffer.
        The return Value is the position in the list order, 0 being the most
  recently used item. You can also determine the registry Value where the item
  is stored by specifying a non-null Value for the lpdwRegNum pointer.
  It will be filled in with a  number corresponding to the item's registry name
  (this is  the same as the Value returned by the AddMRUxxx functions).
        if the item could not be found, or there was some other error, the
  functions will return -1.}
  TFindMRUstring = function (hList:THandle;lpszstring:PChar;lpRegNum:Pinteger):Integer;stdcall;
  TFindMRUData = function (hList:THandle;lpData:Pointer;cbData:DWORD;lpRegNum:Pinteger):Integer;stdcall;

var
  CreateMRUList:TCreateMRUList;
  FreeMruList:TFreeMruList;
  AddMrustring:TAddMrustring;
  AddMruData:TAddMruData;
  DelMRUstring:TDelMRUstring;
  EnumMRUList:TEnumMRUList;
  FindMRUstring:TFindMRUstring;
  FindMRUData:TFindMRUData;

implementation

var
 hDll : THandle;

const
  DllName = 'COMCTL32.DLL';

initialization
  hDll := LoadLibrary(DllName);
  if hDll<>0 then
  begin
    CreateMRUList:=GetProcAddress(hDll,PChar(151));
    FreeMruList:=GetProcAddress(hDll,PChar(152));
    AddMrustring:=GetProcAddress(hDll,PChar(153));
    AddMruData:=GetProcAddress(hDll,PChar(167));
    DelMRUstring:=GetProcAddress(hDll,PChar(156));
    EnumMRUList:=GetProcAddress(hDll,PChar(154));
    FindMRUstring:=GetProcAddress(hDll,PChar(155));
    FindMRUData:=GetProcAddress(hDll,PChar(169));
  end
  else
    PError('MRU');
finalization
  if hDll<>0 then
     FreeLibrary(hDll);
end.
