{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvImageRotate.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvImageRotate;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls, JvTypes;

type
  TJvImageThread = class(TThread)
  private
    FTag: Integer;
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
    property Tag:Integer read FTag write FTag;
  end;

  TJvImageRotate = class(TImage)
  private
     FRotated,FOriginal:TBitmap;
     Ftimer:TJvImageThread;
     FPosition:Integer;
     FInterval:cardinal;
     FColor:TColor;
    FRotating: boolean;
     procedure SetPicture(Value: TBItmap);
     procedure SetRotating(Value:boolean);
     procedure Rotate(Sender: TObject);
     procedure SetInterval(Value:cardinal);
  protected
  public
     constructor Create(AOwner: TComponent);override;
     destructor Destroy;override;
  published
     property Rotating:boolean read FRotating write SetRotating default false;
     property StartImage:TBitmap read FOriginal write SetPicture;
     property Interval:Cardinal read FInterval write SetInterval default 20;
     property FillColor:TColor read FColor write FColor;
     procedure SetAngle(Value:integer);
  end;

implementation

{************************************************************}
constructor TJvImageRotate.Create(AOwner: TComponent);
begin
   FOriginal:=TBitmap.Create;
   FRotated:=TBItmap.Create;
   FTimer:=TJvImageThread.Create(true);
   FTimer.FreeOnTerminate:=true;
   FTimer.FDelay:=20;
   FTimer.FOnDraw:=Rotate;
   FPosition:=0;
   FInterval:=20;
   FRotating:=false;
   inherited;
   FOriginal.assign(Picture.bitmap)
end;
{************************************************************}
destructor TJvImageRotate.Destroy;
begin
   FOriginal.free;
   FRotated.free;
   FTimer.Terminate;
   inherited;
end;
{************************************************************}
procedure TJvImageRotate.SetPicture(Value: TBitmap);
begin
   Value.PixelFormat:=pf24bit;
   FOriginal.PixelFormat:=pf24bit;
   Picture.Bitmap.PixelFormat:=pf24bit;
   Picture.assign(Value);
   FOriginal.assign(Value);
end;
{************************************************************}
procedure TJvImageRotate.SetRotating(Value:boolean);
begin
   FRotating:=Value;
   if not(csDesigning in ComponentState) then
   begin
      if FRotating then
         FTimer.resume
      else
         FTimer.Suspend;
   end;
end;
{************************************************************}
procedure TJvImageRotate.Rotate(Sender: TObject);
begin
   if self.Ftimer.tag=0 then //0 from left to right
   begin
      if FPosition=181 then
           self.Ftimer.tag:=1
      else
      begin
         dec(Fposition);
         if FPosition<0 then Fposition:=359;
         SetAngle(Fposition);
      end;
   end
   else                      //from right to left
   begin
      if Fposition=179 then
         self.Ftimer.Tag:=0
      else
      begin
         FPosition:=(FPosition+1)mod 360;
         SetAngle(FPosition);
      end;
   end;
end;
{************************************************************}
procedure TJvImageRotate.SetAngle(Value:integer);
var
  i,j,iRotationAxis,iOriginal,iPrime,iPrimeRotated,jRotationAxis,jOriginal,
  jPrime,jPrimeRotated  :  Integer;
  RowOriginal    :  pRGBArray;
  RowRotated     :  pRGBArray;
  Theta,sinTheta,cosTheta       :  DOUBLE;
  R,G,B:byte;
begin
   FRotated.Width  := FOriginal.Width;
   FRotated.Height := FOriginal.Height;
   FRotated.PixelFormat := pf24bit;
   iRotationAxis := FOriginal.Width  div 2;
   jRotationAxis := FOriginal.Height div 2;
   Theta := (-Value)*PI/180;
   sinTheta := SIN(Theta);
   cosTheta := COS(Theta);
   B:=Fcolor mod 256;
   G:=round((Fcolor div 256) mod 256);
   R:=round((Fcolor div 256)div 256);
   for j := FRotated.Height-1 downto 0 do
   begin
      RowRotated  := FRotated.Scanline[j];
      jPrime := 2*(j - jRotationAxis) + 1;
      for i := FRotated.Width-1 downto 0 do
      begin
         iPrime := 2*(i - iRotationAxis) + 1;
         iPrimeRotated := ROUND(iPrime * CosTheta - jPrime * sinTheta);
         jPrimeRotated := ROUND(iPrime * sinTheta + jPrime * cosTheta);
         iOriginal := (iPrimeRotated - 1)div 2 + iRotationAxis;
         jOriginal := (jPrimeRotated - 1)div 2 + jRotationAxis;
         if   (iOriginal >= 0) and (iOriginal <= FOriginal.Width-1) and
              (jOriginal >= 0) and (jOriginal <= FOriginal.Height-1)
         then
         begin
            RowOriginal := FOriginal.Scanline[jOriginal];
            RowRotated[i]  := RowOriginal[iOriginal]
         end
         else
         begin
            RowRotated[i].rgbtBlue  := B;
            RowRotated[i].rgbtGreen := G;
            RowRotated[i].rgbtRed   := R
         end;
      end;
   end;
   Picture.assign(FRotated);
end;
{************************************************************}
procedure TJvImageRotate.SetInterval(Value:cardinal);
begin
   FInterval:=Value;
   Ftimer.FDelay:=Value;
end;
{************************************************************}

///////////////////////////////////////////////////////////
// TJvImageThread
///////////////////////////////////////////////////////////

{************************************************************}
procedure TJvImageThread.Draw;
begin
   if Assigned(FOnDraw) then
      FOnDraw(self);
end;
{************************************************************}
procedure TJvImageThread.Execute;
begin
   while not(terminated) do
   begin
      Sleep(FDelay);
      Synchronize(Draw);
   end;
end;
{************************************************************}
end.
