{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvPatchFile.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvPatchFile;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes;

type
  TJvPatchFile = class(TComponent)
  private
    FEnd: TFileName;
    FStart: TFileName;
    FDiff: TstringList;
    FChange: boolean;
    FPos:Integer;
    FPass:string;
    function Decrypt(Value:Byte):byte;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property StartFile:TFileName read FStart write FStart;
    property EndFile:TFileName read Fend write FEnd;
    property ChangeInFile:boolean read FChange write FChange default true;
    property Differences:TstringList read FDiff write FDiff;
    function Patch(Password:string=''):boolean;
    function IsPatched(FileName:string):boolean;
    function IsPatchable(FileName:string):boolean;
  end;

implementation

{**************************************************}
constructor TJvPatchFile.Create(AOwner: TComponent);
begin
   inherited;
   FDiff:=Tstringlist.Create;
   FChange:=true;
end;
{**************************************************}
function TJvPatchFile.Decrypt(Value: Byte): byte;
begin
   if FPass='' then result:=Value
   else
   begin
      FPos:=(FPos+1)mod length(FPass);
      result:=Value xor byte(FPass[FPos+1]);
   end;
end;
{**************************************************}
destructor TJvPatchFile.Destroy;
begin
   FDiff.free;
   inherited;
end;
{**************************************************}
function TJvPatchFile.IsPatchable(FileName: string): boolean;
var
   f:file of byte;
begin
   result:=false;
   if fileexists(FileName) then
   begin
      assignfile(f,FileName);
      reset(f);
      result:=(FDiff.count>3) and (FileSize(f)=StrToInt(FDiff[2]));
      closefile(f);
   end;
end;
{**************************************************}
function TJvPatchFile.IsPatched(FileName: string): boolean;
var
   f:file of byte;
begin
   result:=false;
   if fileexists(FileName) then
   begin
      assignfile(f,FileName);
      reset(f);
      result:=(FDiff.count>3) and (FileSize(f)=StrToInt(FDiff[3]));
      closefile(f);
   end;
end;
{**************************************************}
function TJvPatchFile.Patch(Password:string): boolean;
var
   i:Integer;
   ind,tmp:longint;
   c:byte;
   b:array [0..65000]of byte;
   t,tend:TFileStream;
   t2:TMemoryStream;
begin
   FPos:=-1;
   FPass:=Password;

   //patch it !:)
   result:=false;
   if (Fdiff.count=0)or(FStart='')or(not(fileExists(FStart))) then
      exit;

   t:=TFileStream.Create(FStart,fmOpenRead or fmShareDenyWrite);
   t2:=TMemoryStream.Create;

   if (FDiff.count>3)and(t.Size=StrToInt(FDiff[2])) then
   begin
      result:=true;
      i:=4;
      while i<FDiff.count do
      begin
         if (length(Fdiff[i])>2)and(pos('|',Fdiff[i])<>0) then
         begin
            ind:=StrToInt(Copy(FDiff[i],1,pos('|',Fdiff[i])-1));
            while ind>65000 do
            begin
               t.Read(b,60000);
               t2.Write(b,60000);
               dec(ind,60000);
            end;
            t.Read(b,ind-1);
            t.read(c,1);
            t2.Write(b,ind-1);
            c:=Byte(Fdiff[i][pos('|',Fdiff[i])+1]);
            c:=Decrypt(c);
            t2.write(c,1);
         end
         else if (Length(Fdiff[i])=1) then
         begin
            //File is greater
            t.Position:=t2.Position;
            while t.position<t.size do
            begin
               ind:=t.read(b,10000);
               t2.write(b,ind);
            end;
            c:=Byte(fdiff[i][1]);
            c:=Decrypt(c);
            t2.write(c,1);
         end
         else if (pos('%',Fdiff[i])=4) then
         begin
            //File is smaller
            ind:=StrToInt(Copy(FDiff[i],pos('%',Fdiff[i])+1,length(FDiff[i])));
            while t.position<ind do
            begin
               tmp:=t.read(b,10000);
               if tmp+t.position>ind then
                  t2.write(b,t2.position-ind)
               else
                  t2.write(b,tmp);
            end;
         end;
         inc(i);
      end;
      t.free;
      if FChange then
         tend:=TFileStream.Create(FStart,fmCreate or fmShareExclusive)
      else
         tend:=TFileStream.Create(FEnd,fmCreate or fmShareExclusive);
      t2.position:=0;
      tend.CopyFrom(t2,t2.Size);
      tend.Free;
   end
   else t.free;
   t2.free;
end;
{**************************************************}
end.
