{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRadioButton.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvRadioButton;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  StdCtrls, JvPropAutoSave;

type
  TJvRadioButton = class(TRadioButton)
  private
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnRestored: TNotifyEvent;
    FHotTrack: boolean;
    FHotFont: TFont;
    FFontSave: TFont;
    FOver: boolean;
    FAutoSave: TJvAutoSave;
    procedure SetHotFont(const Value: TFont);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
  public
    procedure SetChecked(Value: Boolean); override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Loaded;override;
  published
    property AutoSave: TJvAutoSave read FAutoSave write FAutoSave;
    property HotTrack: boolean read FHotTrack write FHotTrack default false;
    property HotTrackFont: TFont read FHotFont write SetHotFont;
    property HintColor:TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnRestored:TNotifyEvent read FOnRestored write FOnRestored;
  end;

implementation

{**************************************************}
constructor TJvRadioButton.Create(AOwner: TComponent);
begin
  inherited;
  FHotTrack := false;
  FHotFont := TFont.Create;
  FFontSave := TFont.Create;
  FOver := false;
  FColor := clInfoBk;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FAutoSave := TJvAutoSave.Create(self);
end;
{**************************************************}
procedure TJvRadioButton.SetChecked(Value: Boolean);
begin
  inherited;
  FAutoSave.SaveValue(Checked);
end;
{**************************************************}
procedure TJvRadioButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.style:=Params.style or BS_MULTILINE or BS_TOP;
end;
{**************************************************}
procedure TJvRadioButton.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvRadioButton.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvRadioButton.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved:=Application.HintColor;
    Application.HintColor:=FColor;
    if FHotTrack then
    begin
      FFontSave.Assign(Font);
      Font.Assign(FHotFont);
    end;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvRadioButton.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FHotTrack then
      Font.Assign(FFontSave);
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
destructor TJvRadioButton.Destroy;
begin
  FFontSave.Free;
  FHotFont.Free;
  FAutoSave.Free;
  inherited;
end;
{**************************************************}
procedure TJvRadioButton.SetHotFont(const Value: TFont);
begin
  FHotFont.Assign(Value);
end;
{**************************************************}
procedure TJvRadioButton.Loaded;
var
 b: Boolean;
begin
  inherited;
  if (FAutoSave.LoadValue(b))  then
  begin
    Checked := b;
    if Assigned(FOnRestored) then
      FOnRestored(self);
  end;
end;
{**************************************************}
end.
