{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRas32.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvRas32;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry, JvHRas32;

type
  TJvRas32 = class(TComponent)
  private
    FPath: TFileName;
    FPassword: string;
    FDeviceName: string;
    FUsername: string;
    FEntry: string;
    FDevice: string;
    FPhone: string;
    FCallBack: string;
    FDomain: string;
    FConnection:DWord;
    FHandle:THandle;
    FPHandle:THandle;
    RASEvent:Word;
    FIndex: Integer;
    FConnected: boolean;
    FOnAuthProject: TNotifyEvent;
    FOnAuthChangePassword: TNotifyEvent;
    FOnAuthLinkSpeed: TNotifyEvent;
    FOnDisConnected: TNotifyEvent;
    FOnAuthNotify: TNotifyEvent;
    FOnDeviceConnected: TNotifyEvent;
    FOnReAuthenticate: TNotifyEvent;
    FOnAuthAck: TNotifyEvent;
    FOnConnectDevice: TNotifyEvent;
    FOnAuthRetry: TNotifyEvent;
    FOnAuthenticate: TNotifyEvent;
    FOnWaiTFormodemReset: TNotifyEvent;
    FOnOpenPort: TNotifyEvent;
    FOnAuthCallback: TNotifyEvent;
    FOnRetryAuthentication: TNotifyEvent;
    FOnPortOpened: TNotifyEvent;
    FOnWaitForCallBack: TNotifyEvent;
    FOnPrepareForCallback: TNotifyEvent;
    FOnPasswordExpired: TNotifyEvent;
    FOnInteractive: TNotifyEvent;
    FOnConnected: TNotifyEvent;
    FOnAuthenticated: TNotifyEvent;
    FOnAllDevicesConnected: TNotifyEvent;
    FDll:THandle;
    FRasDial : TRasDial;
    FRasEnumConnections : TRasEnumConnections;
    FRasEnumEntries : TRasEnumEntries;
    FRasGetConnectStatus : TRasGetConnectStatus;
    FRasGetErrorstring : TRasGetErrorstring;
    FRasHangUp : TRasHangUp;
    FRasGetEntryDialParams : TRasGetEntryDialParams;
    FRasValidateEntryName : TRasValidateEntryName;
    FRasCreatePhonebookEntry : TRasCreatePhonebookEntry;
    FRasEditPhonebookEntry : TRasEditPhonebookEntry;
    function GetPhoneBook: Tstringlist;
    procedure WndProc(var Msg: TMessage);
    procedure SetIndex(const Value: integer);
    function GetConnected: boolean;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    property DeviceType:string read FDevice;
    property DeviceName:string read FDeviceName;
    property PhoneNumber:string read FPhone write FPhone;
    property Domain:string read FDomain write FDomain;
    property CallBackNumber:string read FCallBack write FCallBack;
  published
    property PhoneBook:Tstringlist read GetPhoneBook;
    property EntryIndex:Integer read FIndex write SetIndex default -1;
    property PhoneBookPath:TFileName read FPath write FPath;
    property Entry:string read FEntry write FEntry;
    property Username:string read FUsername write FUSername;
    property Password:string read FPassword write FPassword;
    property Connected:boolean read GetConnected write FConnected;
    property OnOpenPort:TNotifyEvent read FOnOpenPort write FOnOpenport;
    property OnPortOpened:TNotifyEvent read FOnPortOpened write FOnPortOpened;
    property OnConnectDevice:TNotifyEvent read FOnConnectDevice write FOnConnectDevice;
    property OnDeviceConnected:TNotifyEvent read FOnDeviceConnected write FOnDeviceConnected;
    property OnAllDevicesConnected:TNotifyEvent read FOnAllDevicesConnected write FOnAllDevicesConnected;
    property OnAuthenticate:TNotifyEvent read FOnAuthenticate write FOnAuthenticate;
    property OnAuthNotify:TNotifyEvent read FOnAuthNotify write FOnAuthNotify;
    property OnAuthRetry:TNotifyEvent read FOnAuthRetry write FOnAuthRetry;
    property OnAuthCallback:TNotifyEvent read FOnAuthCallback write FOnAuthCallback;
    property OnAuthChangePassword:TNotifyEvent read FOnAuthChangePassword write FOnAuthChangePassword;
    property OnAuthProject:TNotifyEvent read FOnAuthProject write FOnAuthProject;
    property OnAuthLinkSpeed:TNotifyEvent read FOnAuthLinkSpeed write FOnAuthLinkSpeed;
    property OnAuthAck:TNotifyEvent read FOnAuthAck write FOnAuthAck;
    property OnReAuthenticate:TNotifyEvent read FOnReAuthenticate write FOnReAuthenticate;
    property OnAuthenticated:TNotifyEvent read FOnAuthenticated write FOnAuthenticated;
    property OnPrepareForCallback:TNotifyEvent read FOnPrepareForCallback write FOnPrepareForCallback;
    property OnWaiTFormodemReset:TNotifyEvent read FOnWaiTFormodemReset write FOnWaiTFormodemReset;
    property OnInteractive:TNotifyEvent read FOnInteractive write FOnInteractive;
    property OnRetryAuthentication:TNotifyEvent read FOnRetryAuthentication write FOnRetryAuthentication;
    property OnPasswordExpired:TNotifyEvent read FOnPasswordExpired write FOnPasswordExpired;
    property OnConnected:TNotifyEvent read FOnConnected write FOnConnected;
    property OnDisConnected:TNotifyEvent read FOnDisConnected write FOnDisConnected;
    property OnWaitForCallBack:TNotifyEvent read FOnWaitForCallBack write FOnWaitForCallBack;
    function Dial(index:integer):boolean;
    function HangUp:boolean;
    function CreateNewConnection:boolean;
    function EditConnection(Index:integer):boolean;
  end;

  ERasError = class(Exception)
  end;

implementation

resourcestring
  RC_RasError           =        'RAS : Unable to find RasApi32.dll';
  RC_RasDllName         =        'RASAPI32.DLL';

{**************************************************}
constructor TJvRas32.Create(AOwner: TComponent);
begin
   inherited;
   FPath:='';
   FPassword:='';
   FDeviceName:='';
   FUsername:='';
   FEntry:='';
   FDevice:='';
   FPhone:='';
   FCallBack:='';
   FDomain:='';
   FConnection:=0;
   FHandle:=AllocateHWnd(WndProc);
   RASEvent:=RegisterWindowMessage(RASDialEvent);
   if RASEvent=0 then
      RASEvent:=WM_RASDialEvent;
   if AOwner is TWinControl then FPHandle:=(AOwner as TWinControl).handle
   else FPHandle:=Application.handle;
   Findex:=-1;

   FDll:=LoadLibrary(PChar(RC_RasDllName));
   if Fdll<>0 then
   begin
      FRasDial := GetProcAddress(Fdll,'RasDialA');
      FRasEnumConnections := GetProcAddress(Fdll,'RasEnumConnectionsA');
      FRasEnumEntries := GetProcAddress(Fdll,'RasEnumEntriesA');
      FRasGetConnectStatus := GetProcAddress(Fdll,'RasGetConnectStatusA');
      FRasGetErrorstring := GetProcAddress(Fdll,'RasGetErrorstringA');
      FRasHangUp := GetProcAddress(Fdll,'RasHangUpA');
      FRasGetEntryDialParams := GetProcAddress(Fdll,'RasGetEntryDialParamsA');
      FRasValidateEntryName := GetProcAddress(Fdll,'RasValidateEntryNameA');
      FRasCreatePhonebookEntry := GetProcAddress(Fdll,'RasCreatePhonebookEntryA');
      FRasEditPhonebookEntry := GetProcAddress(Fdll,'RasEditPhonebookEntryA');
   end
   else
    raise ERasError.Create(RC_RasError);
end;
{**************************************************}
function TJvRas32.CreateNewConnection: boolean;
begin
   if @FRasCreatePhonebookEntry<>nil then
     result:=FRasCreatePhonebookEntry(FPHandle,nil)=0
   else
     result:=false;
end;
{**************************************************}
destructor TJvRas32.Destroy;
begin
   if Fdll<>0 then
   begin
      try
        HangUp;
      except
      end;
      FreeLibrary(Fdll);
   end;
   DeallocateHWnd(FHandle);
   inherited
end;
{**************************************************}
function TJvRas32.Dial(index:integer): boolean;
var
  RASDialParams:TRASDialParams;
  R:Dword;
  x:Integer;
begin
   if (FConnection<>0) then
      result:=false
   else
   begin
      FillChar(RASDialParams, SizeOf(RASDialParams), #0);
      FConnection:=0;
      with RASDialParams do
      begin
         dwsize:=SizeOf(TRasDialParams);
         StrLCopy(szEntryName,pchar(GetPhoneBook[Index]),RAS_MAXENTRYNAME);
         x:=self.entryindex;
         self.EntryIndex:=index;
         StrLCopy(szUserName,pchar(FUsername),RAS_MAXENTRYNAME);
         StrLCopy(szPassword,pchar(FPassword),RAS_MAXENTRYNAME);
         self.entryindex:=x;
         szDomain:='*';
         szCallbackNumber:='*';
         szPhoneNumber:='';
      end;
      if @FRasDial<>nil then
      begin
        if FPath<>'' then
           r:=FRasDial(nil,Pchar(FPath),@RASDialParams,$FFFFFFFF,FHandle,FConnection)
        else
           r:=FRasDial(nil,nil,@RASDialParams,$FFFFFFFF,FHandle,FConnection);
        result:=r=0;
      end
      else result:=false;
   end;
end;
{**************************************************}
function TJvRas32.EditConnection(Index: integer): boolean;
var
   st:TstringList;
begin
   st:=GetPhonebook;

   if @FRasEditPhonebookEntry<>nil then
   begin
      if index<st.Count then
          result:=FRasEditPhonebookEntry(FPhandle,nil,pchar(st[index]))=0
      else
          result:=false;
   end
   else result:=false;
   st.free;
end;
{**************************************************}
function TJvRas32.GetConnected: boolean;
var
   status:TRASConnStatus;
begin
   if (FConnection<>0) and (@FRasGetConnectStatus<>nil) then
   begin
      Status.dwSize:=sizeof(TRASConnStatus);
      FRasGetConnectStatus(FConnection,@Status);
      result:=Status.rasConnstate=RASCS_Connected;
   end
   else result:=false;
end;
{**************************************************}
function TJvRas32.GetPhoneBook: Tstringlist;
var
  RASEntryName: Array[1..50] Of TRasEntryName;
  I,BufSize,Entries: DWord;
begin
   result:=TstringList.Create;
   RasEntryName[1].dwSize:=SizeOf(RasEntryName[1]);
   BufSize:=SizeOf(RasEntryName);

   if @FRasEnumEntries<>nil then
   begin
      if (FPath<>'') then
         i:=FRasEnumEntries(Nil,Pchar(FPath),@RASEntryName, BufSize, Entries)
      else
         i:=FRasEnumEntries(Nil,Nil,@RASEntryName, BufSize, Entries);
      if (i=0)or(i=ERROR_BUFFER_TOO_SMALL) then
           for i:=1 to entries do
               if (i<51)and(RasEntryName[i].szEntryName[0]<>#0) then
                  result.add(strpas(RasEntryName[i].szEntryName));
   end;
end;
{**************************************************}
function TJvRas32.HangUp: boolean;
begin
   result:=false;
   if (FConnection<>0)and(@FRasHangUp<>nil) then
   begin
      result:=FRasHangUp(FConnection)=0;
      FConnection:=0;
   end;
end;
{**************************************************}
procedure TJvRas32.SetIndex(const Value: integer);
var
   RasDial:TRASDialParams;
   res:LongBool;
begin
   FIndex := Value;

   FEntry:='';
   FUsername:='';
   FPhone:='';
   FDomain:='';
   FCallBack:='';
   FPassword:='';

   if (FIndex>=GetPhoneBook.count)then
   begin
      if(GetPhoneBook.count>0) then FIndex:=0
      else FIndex:=-1;
   end;

   if FIndex<>-1 then
   begin
      FEntry:=GetPhonebook[FIndex];

      FillChar(RasDial,sizeof(TRasDialParams),#0);
      StrLCopy(RasDial.szEntryName,pchar(GetPhoneBook[Findex]),RAS_MAXENTRYNAME);
      RasDial.dwSize:=SizeOf(TRasDialParams);

      if @FRasGetEntryDialParams<>nil then
      begin
         if FRasGetEntryDialParams(nil,RasDial,Res)=0 then
         begin
            with RasDial do
            begin
               FUsername:=strpas(szUsername);
               FPassword:=strpas(szPassword);
               FDomain:=strpas(szDomain);
               FCallBack:=strpas(szCallbackNumber);
               FPhone:=strpas(szPhoneNumber);
            end;
         end;
      end;
   end;
end;
{**************************************************}
procedure TJvRas32.WndProc(var Msg: TMessage);
begin
   if (Msg.Msg=RASEvent)and(FConnection<>0) then
   begin
      case Msg.wParam of
           RASCS_OpenPort : if Assigned(FOnOpenPort) then FOnOpenPort(self);
           RASCS_PortOpened : if Assigned(FOnPortOpened) then FOnPortOpened(self);
           RASCS_ConnectDevice : if Assigned(FOnConnectDevice) then FOnConnectDevice(self);
           RASCS_DeviceConnected : if Assigned(FOnDeviceConnected) then FOnDeviceConnected(self);
           RASCS_AllDevicesConnected : if Assigned(FOnAllDevicesConnected) then FOnAllDevicesConnected(self);
           RASCS_Authenticate : if Assigned(FOnAuthenticate) then FOnAuthenticate(self);
           RASCS_AuthNotify : if Assigned(FOnAuthNotify) then FOnAuthNotify(self);
           RASCS_AuthRetry : if Assigned(FOnAuthRetry) then FOnAuthRetry(self);
           RASCS_AuthCallback : if Assigned(FOnAuthCallback) then FOnAuthCallback(self);
           RASCS_AuthChangePassword : if Assigned(FOnAuthChangePassword) then FOnAuthChangePassword(self);
           RASCS_AuthProject : if Assigned(FOnAuthProject) then FOnAuthProject(self);
           RASCS_AuthLinkSpeed : if Assigned(FOnAuthLinkSpeed) then FOnAuthLinkSpeed(self);
           RASCS_AuthAck : if Assigned(FOnAuthAck) then FOnAuthAck(self);
           RASCS_ReAuthenticate : if Assigned(FOnReAuthenticate) then FOnReAuthenticate(self);
           RASCS_Authenticated : if Assigned(FOnAuthenticated) then FOnAuthenticated(self);
           RASCS_PrepareForCallback : if Assigned(FOnPrepareForCallback) then FOnPrepareForCallback(self);
           RASCS_WaiTFormodemReset : if Assigned(FOnWaiTFormodemReset) then FOnWaiTFormodemReset(self);
           RASCS_Interactive : if Assigned(FOnInteractive) then FOnInteractive(self);
           RASCS_RetryAuthentication : if Assigned(FOnRetryAuthentication) then FOnRetryAuthentication(self);
           RASCS_PasswordExpired : if Assigned(FOnPasswordExpired) then FOnPasswordExpired(self);
           RASCS_Connected    : if Assigned(FOnConnected) then FOnConnected(self);
           RASCS_DisConnected : if Assigned(FOnDisConnected) then FOnDisConnected(self);
           RASCS_WaitForCallBack: if Assigned(FOnWaitForCallBack) then FOnWaitForCallBack(self);
      end;
   end
   else
      DefWindowProc(FHandle,Msg.Msg,Msg.wParam,Msg.lParam);
end;
{**************************************************}

end.
