{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvScrollBar.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvScrollBar;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TJvScrollBar = class(TScrollBar)
  private
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FAutoCtl3d: boolean;
    FOver: boolean;
    procedure SetAutoCtl3d(Value:boolean);
  protected
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
  public
    constructor Create(AOwner: TComponent);override;
  published
    property HotTrack: boolean read FAutoCtl3d write SetAutoCtl3d default false;
    property HintColor: TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter: TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave: TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged: TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange: TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

{**************************************************}
constructor TJvScrollBar.Create(AOwner: TComponent);
begin
  inherited;
  FColor := clInfoBk;
  FOver := false;
  FAutoCtl3d := false;
  Controlstyle := Controlstyle + [csAcceptsControls];
end;
{**************************************************}
procedure TJvScrollBar.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvScrollBar.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvScrollBar.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if FAutoCtl3d then
      ctl3d:=true;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvScrollBar.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FAutoCtl3d then
      ctl3d := false;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
procedure TJvScrollBar.SetAutoCtl3d(Value:boolean);
begin
  FAutoCtl3d := Value;
  if FAutoCtl3d then
    Ctl3d := false;
end;
{**************************************************}
end.
