{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvScrollText.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvScrollText;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvTypes;

type
  TJvScrollThread = class(TThread)
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
  end;

  TJvScrollText = class(TWinControl)
  private
    FMemo:TStaticText;
    FTimerTag:Integer;
    FActive: boolean;
    FDelay: cardinal;
    FPixel: Integer;
    FCurrPos:Integer;
    FSelectable: boolean;
    FDirection: TDirection;
    FScrollSaved:Integer;
    Fstrings: TstringList;
    FDeja:Cardinal;
    FScroll:TJvScrollThread;
    FFont: TFont;
    FStartY:Integer;
    FDown:Boolean;

    procedure SetItems(const Value: TstringList);
    procedure OnScroll(Sender: TObject);
    procedure SetActive(const Value: boolean);
    procedure SetDelay(const Value: cardinal);
    procedure SetPixel(const Value: integer);
    procedure SetDirection(const Value: TDirection);
    procedure CalculeMemo(Sender: TObject);
    function GetAlignMent: TAlignment;
    procedure Setalignment(const Value: TAlignment);
    function GeTColor: TColor;
    procedure SeTColor(const Value: TColor);
    procedure FontChanged(Sender: TObject);
    function GetFont: TFont;
    procedure SetFont(const Value: TFont);
    procedure MouseD(Sender: TObject;Button: TMouseButton; Shift: TShiftState; X,Y: integer);
    procedure MouseM(Sender: TObject;Shift: TShiftState; X, Y: integer);
    procedure MouseU(Sender: TObject;Button: TMouseButton; Shift: TShiftState; X,Y: integer);
  protected
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property TextAlignment:TAlignment read GetAlignMent write Setalignment;
    property Items:TstringList read Fstrings write SetItems;
    property Active:boolean read FActive write SetActive default false;
    property Delay:cardinal read FDelay write SetDelay default 50;
    property ScrollPixels:Integer read FPixel write SetPixel default 1;
    property ScrollDirection:TDirection read FDirection write SetDirection default diBottomToTop;
    property BackgroundColor:TColor read GeTColor write SeTColor;
    property Font:TFont read GetFont write SetFont;
    procedure Pause;
    procedure Unpause;
    procedure Reset;
    property Align;
    property ShowHint;
    property ParentShowHint;
  end;

implementation

resourcestring
   RC_TestText         =     'abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';

{*******************************************************}
procedure TJvScrollThread.Draw;
begin
  if Assigned(FOnDraw) then
    FOnDraw(nil)
end;
{*******************************************************}
procedure TJvScrollThread.Execute;
begin
  while not(Terminated) do
  begin
    SynChronize(Draw);
    Sleep(FDelay);
  end;
end;
{*******************************************************}



{*******************************************************}
constructor TJvScrollText.Create(AOwner: TComponent);
begin
   inherited;
   self.width:=200;
   self.height:=150;
   FActive:=false;
   FDelay:=50;
   FPixel:=1;
   FCurrPos:=0;
   FSelectable:=true;
   FDirection:=diBottomToTop;
   Fstrings:=TstringList.Create;

   FMemo:=TStaticText.Create(self);
   FMemo.parent:=self;
   FMemo.Width:=self.width;
   FMemo.height:=self.height;
   FMemo.Borderstyle:=sbsNone;
   FMemo.TabStop:=false;
   FMemo.Enabled:=FSelectable;
   FMemo.AutoSize:=false;
   FMemo.OnMouseDown:=MouseD;
   FMemo.OnMouseMove:=MouseM;
   FMemo.OnMouseUp:=MouseU;

   FFont:=TFont.Create;
   FFont:=FMemo.Font;
   FFont.OnChange:=FontChanged;

   FTimertag:=0;
   FDown:=False;
   FDeja:=Application.HintPause;

   FScroll:=TJvScrollThread.Create(true);
   FScroll.FreeOnTerminate:=false;
   FScroll.FDelay:=FDelay;
   FScroll.FOnDraw:=OnScroll;
end;
{*******************************************************}
function TJvScrollText.GetFont: TFont;
begin
   result:=FMemo.Font;
end;
{*******************************************************}
procedure TJvScrollText.SetFont(const Value: TFont);
var
  FAl:TAlignment; 
begin
   FMemo.Font.Assign(Value);
   CalculeMemo(self);
   Fal:=FMemo.Alignment;
   if FMemo.Alignment=taCenter then
      FMemo.Alignment:=taLeftJustify
   else
      FMemo.Alignment:=taCenter;
   FMemo.Alignment:=FAl;
end;
{*******************************************************}
procedure TJvScrollText.MouseD(Sender: TObject;Button: TMouseButton; Shift: TShiftState;
  X, Y: integer);
var
 p:TPoint;
begin
   p.x:=x;p.y:=y;
   p:=FMemo.ClientToScreen(p);

   if FDirection in [diTopToBottom,diBottomToTop] then
      FStartY:=p.y
   else
      FStartY:=p.x;
   FScroll.FOnDraw:=nil;
   FDown:=true;
end;
{*******************************************************}
procedure TJvScrollText.MouseM(Sender: TObject;Shift: TShiftState; X, Y: integer);
var
  NewY:Integer;
  p:TPoint;
begin
   if FDown then
   begin
      //if NewY>0, going up, NewY<0, going down
      p.x:=x;p.y:=y;
      p:=FMemo.ClientToScreen(p);
      y:=p.y;
      x:=p.x;

      if FDirection in [diTopToBottom,diBottomToTop] then
      begin
         NewY:=FStartY-Y;
         FStartY:=Y;
         FCurrPos:=FCurrPos-NewY;

         if FCurrPos<-FMemo.Height then
            FCurrPos:=self.height
         else if FCurrPos>self.height then
            FCurrPos:=-FMemo.Height;

         FMemo.Top:=FCurrPos;
      end
      else
      begin
         NewY:=FStartY-x;
         FStartY:=x;
         FCurrPos:=FCurrPos-NewY;

         if FCurrPos<-FMemo.Width then
            FCurrPos:=self.width
         else if FCurrPos>self.Width then
            FCurrPos:=-FMemo.Width;

         FMemo.Left:=FCurrPos;
      end;
   end;
end;
{*******************************************************}
procedure TJvScrollText.MouseU(Sender: TObject;Button: TMouseButton; Shift: TShiftState;
  X, Y: integer);
begin
  FScroll.FOnDraw := OnScroll;
  FDown := False;
end;
{*******************************************************}
destructor TJvScrollText.Destroy;
begin
  FScroll.Terminate;
  while not(FScroll.Terminated) do
    Application.ProcessMessages;
  FScroll.Free;
  Application.HintPause := FDeja;
  Fstrings.Free;
  FMemo.Free;
  inherited;
end;
{*******************************************************}
procedure TJvScrollText.OnScroll(Sender: TObject);
var
   t:Integer;
begin
   //tag=1 pause
   if FTImertag=1 then
   begin
      if FscrollSaved<=0 then
      begin
         SetActive(False);
         FTimertag:=0;
         exit;
      end
      else
      begin
         t:=FScrollSaved;
         dec(FScrollSaved);
      end;
   end
   else if FTimertag=2 then
   begin
      if FScrollSaved>=FPixel then
      begin
         FTimertag:=0;
         t:=FPixel;
      end
      else
      begin
         t:=FScrollSaved;
         inc(FSCrollSaved);
      end;
   end
   else t:=FPixel;

   //tag=2 unpause
   //FDirection

   case FDirection of
        diTopToBottom :
                      begin
                         if FCurrPos>self.height then
                            FCurrPos:=-FMemo.height
                         else
                            FCurrPos:=FCurrPos+t;
                         FMemo.Top:=FCurrPos;
                      end;
        diLeftToRight :
                      begin
                         if -FCurrPos>FMemo.width then
                            FCurrPos:=self.width
                         else
                            FCurrpos:=FCurrPos-t;
                         Fmemo.left:=FCurrPos;
                      end;
        diRightToLeft :
                      begin
                         if FCurrPos>self.width then
                            FCurrPos:=-self.width
                         else
                            FCurrpos:=FCurrPos+t;
                         Fmemo.left:=FCurrPos;
                      end;
        diBottomToTop :
                      begin
                         if -FCurrPos>FMemo.height then
                            FCurrPos:=self.height
                         else
                            FCurrPos:=FCurrPos-t;
                         FMemo.Top:=FCurrPos;
                      end;
   end;
end;
{*******************************************************}
procedure TJvScrollText.Pause;
begin
   if FActive then
   begin
      FScrollSaved:=FPixel;
      FTimertag:=1;
   end;
end;
{*******************************************************}
procedure TJvScrollText.SetActive(const Value: boolean);
begin
   SetItems(Fstrings);
   FActive:=Value;
   if Value then
      FScroll.Resume
   else
      FScroll.Suspend;
end;
{*******************************************************}
procedure TJvScrollText.SetDelay(const Value: cardinal);
begin
   if Value>Fdeja then
      Application.hintpause:=Fdeja
   else if Value>10 then
     Application.HintPause:=Value-10
   else
     Application.hintpause:=abs(Value-1);
   FDelay:=Value;
   FScroll.FDelay:=Value;
end;
{*******************************************************}
procedure TJvScrollText.SetDirection(const Value: TDirection);
begin
   FDirection := Value;
   FMemo.left:=0;
   FMemo.top:=0;
   case FDirection of
        diTopToBottom : FCurrpos:=self.height;
        diLeftToRight : FCurrpos:=-self.width;
        diRightToLeft : FCurrpos:=self.width;
        diBottomToTop : FCurrpos:=-Fmemo.height;
   end;
end;
{*******************************************************}
procedure TJvScrollText.CalculeMemo(Sender: TObject);
var
   i,j:Integer;
   ts:TstringList;
begin
   //calculate the size of the memo (vertically)
   with TCanvas.Create do
   begin
      Handle:=GetDc(0);
      Font.Assign(FMemo.Font);
      j:=0;
      ts:=TstringList.Create;
      ts.Text:=FMemo.Caption;
      for i:=0 to ts.Count-1 do
        try
          if ts[i]<>'' then
             j:=j+TextHeight(ts[i])
          else
             j:=j+TextHeight(RC_TestText);
        except
        end;
      if j<=0 then j:=self.Height;
      FMemo.Height:=j;
      ReleaseDc(0,handle);
      ts.free;
      free;
   end;
   if FMemo.height<self.height then
      FMemo.height:=self.height;
   case FDirection of
        diTopToBottom : FCurrpos:=self.height;
        diLeftToRight : FCurrpos:=-self.width;
        diRightToLeft : FCurrpos:=self.width;
        diBottomToTop : FCurrpos:=-Fmemo.height;
   end;
end;
{*******************************************************}
procedure TJvScrollText.SetItems(const Value: TstringList);
begin
   Fstrings.Text:=Value.Text;
   FMemo.caption:=Value.text;
   CalculeMemo(self);
end;
{*******************************************************}
function TJvScrollText.GeTColor: TColor;
begin
   result:=FMemo.Color;
end;
{*******************************************************}
procedure TJvScrollText.SeTColor(const Value: TColor);
begin
   FMemo.Color:=Value;
   self.color:=Value;
   invalidate;
end;
{*******************************************************}
procedure TJvScrollText.FontChanged(Sender: TObject);
var
  FAl:TAlignment;
begin
   CalculeMemo(self);
   Fal:=FMemo.Alignment;
   if FMemo.Alignment=taCenter then
      FMemo.Alignment:=taLeftJustify
   else
      FMemo.Alignment:=taCenter;
   FMemo.Alignment:=FAl;
end;
{*******************************************************}
procedure TJvScrollText.SetPixel(const Value: integer);
begin
   FPixel := Value;
end;
{*******************************************************}
procedure TJvScrollText.Reset;
begin
   case FDirection of
       diTopToBottom : FCurrpos:=self.height;
       diLeftToRight : FCurrpos:=-self.width;
       diRightToLeft : FCurrpos:=self.width;
       diBottomToTop : FCurrpos:=-Fmemo.height;
   end;
end;
{*******************************************************}
procedure TJvScrollText.Unpause;
begin
   if not(Factive)then
   begin
      FScrollSaved:=0;
      FTimertag:=2;
      SetActive(true);
   end;
end;
{*******************************************************}
procedure TJvScrollText.WMSize(var Message: TWMSize);
begin
   FMemo.Width:=self.width;
   if FMemo.Height<self.height then FMemo.height:=self.height;
end;
{*******************************************************}
function TJvScrollText.GetAlignMent: TAlignment;
begin
   result:=FMemo.Alignment;
end;
{*******************************************************}
procedure TJvScrollText.Setalignment(const Value: TAlignment);
begin
   FMemo.Alignment:=Value;
end;
{*******************************************************}

end.
