{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvTreeView2.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvTreeView2;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Menus,
  ComCtrls, JvTypes, CommCtrl;

type
  TJvTreeNode = class (TTreeNode)
  private
    FBold:boolean;
    FChecked:Boolean;
    FPopupMenu: TPopupMenu;
    function GetChecked :Boolean;
    procedure SetChecked(Value:Boolean);
    function GetBold: boolean;
    procedure SetBold(const Value: boolean);
    procedure SetPopupMenu(const Value: TPopupMenu);
  public
    constructor CreateEnh(AOwner: TTreeNodes);
    property Checked:Boolean read GetChecked write SetChecked;
    property Bold:boolean read GetBold write SetBold;
    property PopupMenu:TPopupMenu read FPopupMenu write SetPopupMenu;
  end;

  TPageChangedEvent = procedure (Sender: TObject; Item:TTreeNode; Page:TTabSheet) of object;

  TJvTreeView2 = class(TTreeView)
  private
    FColor,FSaved:TColor;
    FOnMouseEnter : TNotifyEvent;
    FOnMouseLeave : TNotifyEvent;
    FOnParentColorChanged : TNotifyEvent;
    FOnCtl3DChanged : TNotifyEvent;
    FOver:boolean;
    FCheckBoxes: boolean;
    FOnHScroll: TNotifyEvent;
    FOnVScroll: TNotifyEvent;
    FMenu: TPageControl;
    FOnPage: TPageChangedEvent;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure MouseEnter(var msg:Tmessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg:Tmessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorChangeD;
    procedure SetCheckBoxes(const Value: boolean);
  protected
    function CreateNode: TTreeNode; override;
    procedure CreateParams(var Params: TCreateParams);override;
    procedure WMNOTIFY(var msg : TWMNotify);message CN_NOTIFY;
  public
    constructor Create(AOwner: TComponent);override;
  published
    property HintColor:TColor read FColor write FColor default clInfoBk;
    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property Checkboxes : boolean read FCheckBoxes write SetCheckBoxes default false;
    function GetBold(Node:TTreeNode):boolean;
    procedure SetBold(Node:TTreeNode;Value:boolean);
    function GetChecked(Node:TTreenode):boolean;
    procedure SetChecked(Node:TTreenode;Value:boolean);
    procedure SetNodePopup(Node:TTreeNode;Value:TPopupMenu);
    function GetNodePopup(Node:TTreeNode):TPopupMenu;
    property OnVerticalScroll:TNotifyEvent read FOnVScroll write FOnVScroll;
    property OnHorizontalScroll:TNotifyEvent read FOnHScroll write FOnHScroll;
    property Menu:TPageControl read FMenu write FMenu;
    property OnPageChanged:TPageChangedEvent read FOnPage write FOnPage;
  end;

implementation

{***********************************************}
constructor TJvTreeNode.CreateEnh(AOwner: TTreeNodes);
begin
   inherited Create(AOwner);
   FPopupMenu:=TPopupMenu.Create(AOwner.Owner);
end;
{***********************************************}
procedure TJvTreeNode.SetPopupMenu(const Value: TPopupMenu);
begin
   FPopupMenu:=Value;
end;
{***********************************************}
function TJvTreeNode.GetBold: boolean;
var
  Item: TTVItem;
begin
   with Item do
   begin
      mask:=TVIF_STATE;
      hItem:=ItemId;
      if TreeView_GetItem(Handle, Item) then
         result:=((Item.State and TVIS_BOLD)=TVIS_BOLD)
      else
         result:=false;
   end;
end;
{***********************************************}
function TJvTreeNode.GetChecked: Boolean;
var
  Item: TTVItem;
begin
   with Item do
   begin
      mask:=TVIF_STATE;
      hItem:=ItemId;
      if TreeView_GetItem(Handle, Item) then
         result:=((Item.State and TVIS_CHECKED)=TVIS_CHECKED)
      else
         result:=false;
   end;
end;
{***********************************************}
procedure TJvTreeNode.SetBold(const Value: boolean);
var
  Item: TTVItem;
begin
   FBold:=Value;
   FillChar(Item,SizeOf(Item),0);
   with Item do
   begin
      mask := TVIF_STATE;
      hItem := ItemId;
      StateMask:=TVIS_BOLD;
      if FBold then
        Item.State:=TVIS_BOLD
      else
        Item.State:=0;
      TreeView_SetItem(Handle, Item);
   end;
end;
{***********************************************}
procedure TJvTreeNode.SetChecked(Value: Boolean);
var
  Item: TTVItem;
begin
   FChecked:=Value;
   FillChar(Item,SizeOf(Item),0);
   with Item do
   begin
      hItem:=ItemId;
      mask:=TVIF_STATE;
      StateMask:=TVIS_STATEIMAGEMASK;
      if FChecked then
        Item.State:=TVIS_CHECKED
      else
        Item.State:=TVIS_CHECKED shr 1;
      TreeView_SetItem(Handle, Item);
   end;
end;
{***********************************************}

///////////////////////////////////////////////////////////
// TJvTreeView2
///////////////////////////////////////////////////////////

{***********************************************}
constructor TJvTreeView2.Create(AOwner: TComponent);
begin
   FColor:=clInfoBk;
   FOver:=false;
   inherited Create(AOwner);
   FCheckBoxes:=false;
   Controlstyle := Controlstyle + [csAcceptsControls];
end;
{***********************************************}
procedure TJvTreeView2.CreateParams(var Params: TCreateParams);
begin
   inherited;
   if FCheckBoxes then
      Params.style:=Params.style or TVS_CHECKBOXES;
end;
{***********************************************}
function TJvTreeView2.CreateNode: TTreeNode;
begin
   Result:=TJvTreeNode.CreateEnh(Items);
end;
{***********************************************}
procedure TJvTreeView2.WMNOTIFY(var msg : TWMNotify);
var
  Node:TTreeNode;
  Point:TPoint;
  i,j:Integer;
begin
   inherited;

   Point:=Mouse.CursorPos;
   Point:=ScreenToClient(point);
   with Msg,point do
      case NMHdr^.code of
           NM_CLICK,NM_RCLICK : begin
                                   Node:=GetNodeAt(x,y);
                                   if Assigned(Node) then
                                      Selected:=Node
                                   else
                                   begin
                                      if FCheckBoxes then
                                      begin
                                         Node:=GetNodeAt(x+16,y);
                                         if Assigned(Node) then
                                            Selected:=Node
                                      end;
                                   end;
                                   if (Selected<>nil)and(NMHdr^.code=NM_RCLICK) then
                                        TJvTreeNode(selected).PopupMenu.Popup(Mouse.CursorPos.x,Mouse.CursorPos.Y);
                                end;
           TVN_SELCHANGEDA, TVN_SELCHANGEDW :
                                begin
                                  if Assigned(FMenu) then
                                    if self.Selected<>nil then
                                    begin
                                      //Search for the correct page
                                      j:=-1;
                                      for i:=0 to FMenu.PageCount-1 do
                                        if FMenu.Pages[i].Caption=self.Selected.Text then
                                           j:=i;
                                      if j<>-1 then
                                      begin
                                        FMenu.ActivePage:=FMenu.Pages[j];
                                        if Assigned(FOnPage) then
                                           FOnPage(self, self.Selected, FMenu.Pages[j]);
                                      end;
                                    end;
                                end;

      end;
end;
{**************************************************}
procedure TJvTreeView2.WMHScroll(var Message: TWMHScroll);
begin
   inherited;
   if Assigned(FOnHScroll) then
      FOnHScroll(self);
end;
{**************************************************}
procedure TJvTreeView2.WMVScroll(var Message: TWMVScroll);
begin
   inherited;
   if Assigned(FOnVScroll) then
      FOnVScroll(self);
end;
{***********************************************}
procedure TJvTreeView2.MouseEnter(var msg:Tmessage);
begin
   FOver:=true;
   FSaved:=Application.HintColor;
   Application.HintColor:=FColor;
   if Assigned(FonMouseEnter) then FonMouseEnter(self);
end;
{**************************************************}
procedure TJvTreeView2.MouseLeave(var msg:Tmessage);
begin
   Application.HintColor:=FSaved;
   FOver:=false;
   if Assigned(FonMouseLeave) then FonMouseLeave(self);
end;
{**************************************************}
procedure TJvTreeView2.CMCtl3DChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonCtl3DChanged) then FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvTreeView2.CMParentColorChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonParentColorChanged) then FonParentColorChanged(Self);
end;
{**************************************************}
function TJvTreeView2.GetBold(Node: TTreeNode): boolean;
begin
   result:=TJvTreeNode(Node).Bold;
end;
{**************************************************}
procedure TJvTreeView2.SetBold(Node: TTreeNode; Value: boolean);
begin
   TJvTreeNode(Node).Bold:=Value;
end;
{**************************************************}
function TJvTreeView2.GetChecked(Node: TTreenode): boolean;
begin
   result:=TJvTreeNode(Node).Checked;
end;
{**************************************************}
procedure TJvTreeView2.SetChecked(Node: TTreenode; Value: boolean);
begin
   TJvTreeNode(Node).Checked:=Value;
end;
{**************************************************}
procedure TJvTreeView2.SetCheckBoxes(const Value: boolean);
begin
   FCheckBoxes := Value;
   ReCreateWnd;
end;
{**************************************************}
function TJvTreeView2.GetNodePopup(Node: TTreeNode): TPopupMenu;
begin
   result:=TJvTreeNode(Node).PopupMenu;
end;
{**************************************************}
procedure TJvTreeView2.SetNodePopup(Node: TTreeNode; Value: TPopupMenu);
begin
   TJvTreeNode(Node).PopupMenu:=Value;
end;
{**************************************************}
end.
